/**
   This class implements rectangles with immutable perimeter.
<p>
   When a {@code RestrictedRectangle} object is created
   the constructor is passed two parameters, {@code width}
   and {@code height}. But the object instead has the two
   fields height and perimeter. The perimeter field is computed
   from the width and height constructor parameters.
<p>
   Once the perimeter is set, it can't be changed (it
   becomes "immutable"). The other "fields" (width and
   height) can only be changed relative to the restriction
   of keeping the perimeter constant (hence the name of
   the class, RestrictedRectangle). If width is increased,
   height must be decreased to keep the perimeter constant.
<p>
   The public interface appears to have {@code width} and
   {@code height} fields, but the private implementation has
   height and perimeter fields. It is important to realize that
   the public get and set methods that a class provides don't
   need to be exactly like the actual fields hidden in the
   private implementation of the class.
<p>
   Use DrJava's interactions pane (the Java interpreter)
   to experiment with RestrictedRectangle objects.
*/

public class RestrictedRectangle
{
   private final double perimeter; // perimeter can be set only one time
   private double height;          // we could use width instead of height

   /**
      This constructor lets you specify the initial width and height of a new
      {@code RestrictedRectangle} object. Specifying the width and height also
      specifies this rectangle's perimeter, which then becomes an invariant of
      this object. That is, the perimeter is immutable. If you later change the
      width of this rectangle, the rectangle's height must then be adjusted to
      keep the perimeter constant. If you change the height of this rectangle,
      the rectangle's width must be adjusted.
      <p>
      The {@code width} and {@code height} cannot be negative.

      @param width   the initial width for the new {@code RestrictedRectangle} object
      @param height  the initial height for the new {@code RestrictedRectangle} object
   */
   public RestrictedRectangle(double width, double height)
   {
      if (height <= 0 || width <= 0)
         throw new IllegalArgumentException("Illegal dimension");

      this.perimeter = 2*width + 2*height; // fill in the "blank final"
      this.height = height;
   }

   /**
      Changing this {@code RestrictedRectangle}'s height causes its
      width to be adjusted so that the perimeter remains constant.
      <p>
      The new {@code height} cannot be negative and it cannot be more than half the perimeter.

      @param height  the new height for the {@code RestrictedRectangle} object
   */
   public void setHeight(double height)
   {
      if (height <= 0)
         throw new IllegalArgumentException("height cannot be negative");

      if (height > perimeter/2)
         throw new IllegalArgumentException("height exceeds half of perimeter");

      this.height = height;
   }

   /**
      Changing this {@code RestrictedRectangle}'s width causes its
      height to be adjusted so that the perimeter remains constant.
      <p>
      The new {@code width} cannot be negative and it cannot be more than half the perimeter.

      @param width  the new width for the {@code RestrictedRectangle} object
   */
   public void setWidth(double width)
   {
      if (width <= 0)
         throw new IllegalArgumentException("width cannot be negative");

      if (width > perimeter/2)
         throw new IllegalArgumentException("width exceeds half of perimeter");

      setHeight( (this.perimeter - 2*width)/2 );
   }

   /**
      @return the height of this {@code RestrictedRectangle} object
   */
   public double getHeight(){ return height; }

   /**
      @return the width of this {@code RestrictedRectangle} object
   */
   public double getWidth(){ return (perimeter - 2*height)/2; }

   /**
      @return the area of this {@code RestrictedRectangle} object
   */
   public double getArea(){ return getWidth() * getHeight(); }

   /**
      @return the immutable perimeter of this {@code RestrictedRectangle} object
   */
   public double getPerimeter(){ return perimeter; }

   /**
      @return {@link String} representation of this {@code RestrictedRectangle} object
   */
   @Override
   public String toString()
   {
      return "RestrictedRectangle" + ": width = " + getWidth()
                                   + ", height = " + getHeight();
   }
}
