/**
    This class encapsulates the data from a temperature reading.
    The details of how a temperature is stored are hidden from the
    user of objects created from this class. The public interface of
    this class requires that the scale of a temperature must always
    be specified. This provides a safe way to store and retrieve
    temperature values.

    @author Roger Kraft
    @version 2
*/

/*
   A temperature is stored in degrees Kelvin, but this
   implementation detail is hidden from the user of
   TemperatureReading2 objects by the public interface
   (i.e., the constructors and the set and get methods).
   The public interface uses degrees Celsius and Fahrenheit.

   The design of this class makes it almost impossible to
   mistake a Celsius value for a Fahrenheit value.
*/
public class TemperatureReading2
{
   // three instance variables
   private double degreesKelvin;
   private double latitude;
   private double longitude;

   /**
      The default constructor.
   */
   public TemperatureReading2()
   {
      degreesKelvin = 0;
      latitude = 0;
      longitude = 0;
   }


   /**
      A constructor that specifies the scale for the temperature.

      @param temperature  the temperature in either Celsius or Fahrenheit
      @param scale        specifies the scale for {@code temperature}, must be either 'C' or 'F'
      @param latitude     the latitude of the reading
      @param longitude    the longitude of the reading
   */
   // Why can't we have TemperatureReading2F and
   // TemperatureReading2C constructors for each
   // temperature scale?
   public TemperatureReading2(double temperature, char scale,
                              double latitude,
                              double longitude)
   {
      if (scale == 'C' )
      {
         degreesKelvin = temperature + 273;
      }
      else if (scale == 'F')
      {
         degreesKelvin = (temperature - 32)/1.8 + 273;
      }
      else
         throw new IllegalArgumentException("Illegal temperature scale");

      // now validate the data so that there cannot be an
      // unrealistic value for the temperature
      if ( degreesKelvin < 0 )
         throw new IllegalArgumentException("Illegal temperature");

      // now initialize the other two fields
      this.latitude = latitude;
      this.longitude = longitude;
   }


   /**
      Retrieve the temperature value using the Celsius scale.

      @return  the temperature of this reading as a value in degrees Celsius
   */
   public double getTemperatureC()
   {
      return degreesKelvin - 273;
   }


   /**
      Retrieve the temperature value using the Fahrenheit scale.

      @return  the temperature of this reading as a value in degrees Fahrenheit
   */
   public double getTemperatureF()
   {
      return (degreesKelvin - 273)*1.8 + 32;
   }


   /**
      Retrieve the latitude.

      @return  the latitude of this temperature reading
   */
   public double getLatitude()
   {
      return latitude;
   }


   /**
      Retrieve the longitude.

      @return  the longitude of this temperature reading
   */
   public double getLongitude()
   {
      return longitude;
   }


   /**
      Set the temperature value using the Celsius scale.

      @param degreesC  new temperature value in degrees Celsius
   */
   public void setTemperatureC(double degreesC)
   {
      degreesKelvin = degreesC + 273;

      if ( degreesKelvin < 0 )  //validate the data
         throw new IllegalArgumentException("Illegal temperature");
   }


   /**
      Set the temperature value using the Fahrenheit scale.

      @param degreesF  new temperature value in degrees Fahrenheit
   */
   public void setTemperatureF(double degreesF)
   {
      degreesKelvin = (degreesF - 32)/1.8 + 273;

      if ( degreesKelvin < 0 )  //validate the data
         throw new IllegalArgumentException("Illegal temperature");
   }


   /**
      @param  latitude value for this {@code TemperatureReading4} object
   */
   public void setLatitude(double latitude)
   {
      this.latitude = latitude;
   }


   /**
      @param  longitude value for this {@code TemperatureReading4} object
   */
   public void setLongitude(double longitude)
   {
      this.longitude = longitude;
   }


   /**
      @return {@link String} representation of this {@code TemperatureReading2} object
   */
   @Override
   public String toString()
   {
      return "TemperatureReading2: " + getTemperatureF() + "F, "
                                     + getTemperatureC() + "C";
   }
}
