
import java.util.Date;

/**
    This class encapsulates the data from a temperature reading.
    The details of how a temperature is stored are hidden from the
    user of objects created from this class. The public interface of
    this class requires that the scale of a temperature must always
    be specified. This provides a safe way to store and retrieve
    temperature values.

    @author Roger Kraft
    @version 5
*/

/*
   The main difference between this class and the
   TemperatureReading4 class is that this class uses
   static factory methods instead of a constructor.
*/
public class TemperatureReading5
{
   // four instance variables (the last of which is
   // itself an object, a Date object)
   private double degreesKelvin;
   private double latitude;
   private double longitude;
   private Date   timeOfReading;

   // some static variables (these don't have to
   // be declared final, but it is a good idea to)
   public static final double BOILING_C = 100.0;
   public static final double BOILING_F = 212.0;
   public static final double FREEZING_C = 0.0;
   public static final double FREEZING_F = 32.0;

   /**
      A private constructor for use by the two static factory methods.

      @param degreesKelvin  the temperature in Kelvin
      @param latitude       the latitude of the reading
      @param longitude      the longitude of the reading
   */
   private TemperatureReading5(double degreesKelvin,
                               double latitude,
                               double longitude)
   {
      // validate the data so that there cannot be an
      // unrealistic value for the temperature
      if ( degreesKelvin < 0 )
         throw new IllegalArgumentException("Illegal temperature");

      this.degreesKelvin = degreesKelvin;
      this.latitude = latitude;
      this.longitude = longitude;
      // Construct a new Date object. According to the
      // Java documentation, the default Date constructor
      // creates a Date object with the time of its creation.
      timeOfReading = new Date();
   }


   /**
      Construct a TemperatureReading5 object from a degrees Celsius value.
      <p>
      This is a "static factory method".
      <p>
      This method is used in place of a public constructor.
      It's main advantage is that it can be given a more
      descriptive name and does not need a "temperature
      scale" parameter.

      @param degreesC   the temperature in degrees Celsius
      @param latitude   the latitude of the reading
      @param longitude  the longitude of the reading
      @return a {@code TemperatureReading5} object holding the correct temperature
   */
   public static TemperatureReading5 makeTempC(double degreesC,
                                               double latitude,
                                               double longitude)
   {
      double degreesKelvin = degreesC + 273;

      return new TemperatureReading5(degreesKelvin, latitude, longitude);
   }


   /**
      Construct a TemperatureReading5 object from a degrees Fahrenheit value.
      <p>
      This is a "static factory method".
      <p>
      This method is used in place of a public constructor.
      It's main advantage is that it can be given a more
      descriptive name and does not need a "temperature
      scale" parameter.

      @param degreesF   the temperature in degrees Fahrenheit
      @param latitude   the latitude of the reading
      @param longitude  the longitude of the reading
      @return a {@code TemperatureReading5} object holding the correct temperature
   */
   public static TemperatureReading5 makeTempF(double degreesF,
                                               double latitude,
                                               double longitude)
   {
      double degreesKelvin = (degreesF - 32)/1.8 + 273;

      return new TemperatureReading5(degreesKelvin, latitude, longitude);
   }


   /**
      Retrieve the temperature value using the Celsius scale.

      @return  the temperature of this reading as a value in degrees Celsius
   */
   public double getTemperatureC()
   {
      return degreesKelvin - 273;
   }


   /**
      Retrieve the temperature value using the Fahrenheit scale.

      @return  the temperature of this reading as a value in degrees Fahrenheit
   */
   public double getTemperatureF()
   {
      return (degreesKelvin - 273)*1.8 + 32;
   }


   /**
      Retrieve the latitude.

      @return  the latitude of this temperature reading
   */
   public double getLatitude()
   {
      return latitude;
   }


   /**
      Retrieve the longitude.

      @return  the longitude of this temperature reading
   */
   public double getLongitude()
   {
      return longitude;
   }


   /**
      Retrieve the time of the reading.

      @return  the time of this temperature reading
   */
   public Date getTimeOfReading()
   {
      return timeOfReading;
   }


   /**
      Set the temperature value using the Celsius scale.

      @param degreesC  new temperature value in degrees Celsius
   */
   public void setTemperatureC(double degreesC)
   {
      degreesKelvin = degreesC + 273;

      if ( degreesKelvin < 0 )  //validate the data
         throw new IllegalArgumentException("Illegal temperature");
   }


   /**
      Set the temperature value using the Fahrenheit scale.

      @param degreesF  new temperature value in degrees Fahrenheit
   */
   public void setTemperatureF(double degreesF)
   {
      degreesKelvin = (degreesF - 32)/1.8 + 273;

      if ( degreesKelvin < 0 )  //validate the data
         throw new IllegalArgumentException("Illegal temperature");
   }


   /**
      @param  latitude value for this {@code TemperatureReading5} object
   */
   public void setLatitude(double latitude)
   {
      this.latitude = latitude;
   }


   /**
      @param  longitude value for this {@code TemperatureReading5} object
   */
   public void setLongitude(double longitude)
   {
      this.longitude = longitude;
   }


   /**
      @param date  {@link Date} object for this {@code TemperatureReading5} object
   */
   public void setTimeOfReading(Date date)
   {
      timeOfReading = date;
   }


   /**
      @return {@link String} representation of this {@code TemperatureReading5} object
   */
   @Override
   public String toString()
   {
      return "TemperatureReading5: " + getTemperatureF() + "F, "
                                     + getTemperatureC() + "C @ "
                                     + getTimeOfReading();
   }


   /**
      A static convenience method that converts numbers from a Celsius value to a Fahrenheit value.

      @param degreesC  a number that is interpreted as a temperature in degrees Celsius
      @return  the computed Fahrenheit temperature
   */
   public static double convertCtoF(double degreesC)
   {
      return (degreesC*1.8)+32;
   }


   /**
      A static convenience method that converts numbers from a Fahrenheit value to a Celsius value.

      @param degreesF  a number that is interpreted as a temperature in degrees Fahrenheit
      @return  the computed Celsius temperature
   */
   public static double convertFtoC(double degreesF)
   {
      return (degreesF - 32)/1.8;
   }
}
