
import java.util.*;

/**
    This abstract class encapsulates the data from a temperature reading.
    The details of how a temperature is stored are hidden from the
    user of objects created from this class. The public interface of
    this class requires that the scale of a temperature must always
    be specified. This provides a safe way to store and retrieve
    temperature values.

    @author Roger Kraft
    @version 6
*/

/*
   This class defines an abstract base class for temperature
   reading objects. Since this class is abstract, it cannot be
   used to instantiate actual objects.

   Each subclass should provide a constructor that lets you use
   a specific temperature scale to instantiate a temperature
   reading object

   The subclasses to this class only need to provide a constructor
   (for one temperature scale). All the rest of the code for the
   interface to temperature reading objects is in this base class.
*/
public abstract class TemperatureReading6
{
   // four instance variables (the last of which is
   // itself an object, a Date object)
   private double degreesKelvin;
   private double latitude;
   private double longitude;
   private Date   timeOfReading;

   // some static variables (these don't have to
   // be declared final, but it is a good idea to)
   public static final double BOILING_C = 100.0;
   public static final double BOILING_F = 212.0;
   public static final double FREEZING_C = 0.0;
   public static final double FREEZING_F = 32.0;

   /**
      The default constructor.
   */
   public TemperatureReading6()
   {
      degreesKelvin = 0;
      latitude = 0;
      longitude = 0;
      // Construct a new Date object. According to the
      // Java documentation, the default Date constructor
      // creates a Date object with the time of its creation.
      timeOfReading = new Date();
   }

   /**
      A constructor that specifies the temperature (in Kelvin) and the location.

      @param degreesKelvin  the temperature in degrees Kelvin
      @param latitude       the latitude of the reading
      @param longitude      the longitude of the reading
   */
   protected TemperatureReading6(double degreesKelvin,
                                 double latitude,
                                 double longitude)
   {
      // validate the data so that there cannot be an
      // unrealistic value for the temperature
      if ( degreesKelvin < 0 )
         throw new IllegalArgumentException("Illegal temperature");

      this.degreesKelvin = degreesKelvin;

      // now initialize the latitude and longitude fields
      this.latitude = latitude;
      this.longitude = longitude;
      // construct a new Date object
      timeOfReading = new Date();
   }


   /**
      Retrieve the temperature value using the Celsius scale.

      @return  the temperature of this reading as a value in degrees Celsius
   */
   public double getTemperatureC()
   {
      return degreesKelvin - 273;
   }


   /**
      Retrieve the temperature value using the Fahrenheit scale.

      @return  the temperature of this reading as a value in degrees Fahrenheit
   */
   public double getTemperatureF()
   {
      return (degreesKelvin - 273)*1.8 + 32;
   }


   /**
      Retrieve the latitude.

      @return  the latitude of this temperature reading
   */
   public double getLatitude()
   {
      return latitude;
   }


   /**
      Retrieve the longitude.

      @return  the longitude of this temperature reading
   */
   public double getLongitude()
   {
      return longitude;
   }


   /**
      Retrieve the time of the reading.

      @return  the time of this temperature reading
   */
   public Date getTimeOfReading()
   {
      return timeOfReading;
   }


   /**
      Set the temperature value using the Celsius scale.

      @param degreesC  new temperature value in degrees Celsius
   */
   public void setTemperatureC(double degreesC)
   {
      if (degreesC >= -273)  //validate the data
          degreesKelvin = degreesC + 273;
      else
          degreesKelvin = 0;
   }


   /**
      Set the temperature value using the Fahrenheit scale.

      @param degreesF  new temperature value in degrees Fahrenheit
   */
   public void setTemperatureF(double degreesF)
   {
      degreesKelvin = (degreesF - 32)/1.8 + 273;
      //now validate the data
      if ( degreesKelvin < 0 ) degreesKelvin = 0;
   }


   // set methods for latitude, longitude, and timeOfReading
   public void setLatitude(double latitude)
   {
      this.latitude = latitude;
   }


   public void setLongitude(double longitude)
   {
      this.longitude = longitude;
   }


   public void setTimeOfReading(Date date)
   {
      timeOfReading = date;
   }


   /**
      @return {@link String} representation of this {@code TemperatureReading6} object
   */
   @Override
   public String toString()
   {
      return "TemperatureReading6: " + getTemperatureF() + "F, "
                                     + getTemperatureC() + "C @ "
                                     + getTimeOfReading();
   }


   /**
      A static convenience method that converts numbers from a Celsius value to a Fahrenheit value.

      @param degreesC  a number that is interpreted as a temperature in degrees Celsius
      @return  the computed Fahrenheit temperature
   */
   public static double convertCtoF(double degreesC)
   {
      return (degreesC*1.8)+32;
   }


   /**
      A static convenience method that converts numbers from a Fahrenheit value to a Celsius value.

      @param degreesF  a number that is interpreted as a temperature in degrees Fahrenheit
      @return  the computed Celsius temperature
   */
   public static double convertFtoC(double degreesF)
   {
      return (degreesF - 32)/1.8;
   }
}
